/*

  This function will conduct SExtractor photometry for a given image.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <gsl/gsl_statistics.h>
#include <gsl/gsl_sort.h>

#include <sys/types.h> // for getpid()
#include <unistd.h> // also for getpid() and unlink() ... 

#include "limits.h"
#include "ident.h"

int find_catalog_in_vast_images_catalogs_log(char *fitsfilename, char *catalogfilename){
 char fitsfilename_to_test[FILENAME_LENGTH];
 FILE *f;
 f=fopen("vast_images_catalogs.log","r");
 if(f==NULL){
  strcpy(catalogfilename,"image00000.cat");
  return 1; // not only this image has not been processed, even "vast_images_catalogs.log" is not created yet!
 }
 int found=0;
 while(-1<fscanf(f,"%s %s",catalogfilename,fitsfilename_to_test))
  if(0==strcmp(fitsfilename_to_test,fitsfilename)){found=1;break;}
 fclose(f);
 if( found==0 ){
  strcpy(catalogfilename,"image00000.cat");
  return 1; // it is possible that image00000.cat is referring to another image, so we'll recompute...
 }
 /* Check if the catalog already exist */
 f=fopen(catalogfilename,"r");
 if(f==NULL)
  return 1;
 else
  fclose(f);
 return 0;
}

double autodetect_aperture(char *fitsfilename, char *output_sextractor_catalog, int force_recompute, int param_P, double fixed_aperture, int verbose, double X_im_size, double Y_im_size){
         
	char command[1024];
        double *A;
	int i = 0;
	double median_A;
	double APERTURE;
	int sextractor_flag;


	char aperture_filename[FILENAME_LENGTH]; // this file simply stores the aperture size
	FILE *aperture_file;

        /* Update PATH variable to make sure the local copy of SExtractor is there */
        char pathstring[8192];
        strncpy(pathstring,getenv("PATH"),8192);
        strcat(pathstring,":lib/bin");                            
        setenv("PATH", pathstring, 1);

        /* Check if the file has already been processed */
        if(0==find_catalog_in_vast_images_catalogs_log( fitsfilename, output_sextractor_catalog)){
         sprintf(aperture_filename,"%s.aperture",output_sextractor_catalog);
         aperture_file=fopen(aperture_filename,"r");
         if(NULL==aperture_file){fprintf(stderr,"ERROR: cannot open %s\n",aperture_filename);exit(1);}
         fscanf(aperture_file,"%lf\n",&APERTURE);
         fclose(aperture_file);
         if(0==force_recompute){
          return APERTURE;
         }
        }
        sprintf(aperture_filename,"%s.aperture",output_sextractor_catalog);

        /* Reading file which defines rectangular regions we want to exclude */
        //float cpgline_tmp_x[2];
        //float cpgline_tmp_y[2];
        int N_bad_regions = 0;
        double *X1;
        double *Y1;
        double *X2;
        double *Y2;

        int pid=getpid();
	char sextractor_catalog_filename[FILENAME_LENGTH];
	char psf_filename[FILENAME_LENGTH];
	

        
        X1=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( X1==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for X1\n");exit(1);}
        Y1=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( Y1==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for Y1\n");exit(1);}
        X2=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( X2==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for X2\n");exit(1);}
        Y2=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( Y2==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for Y2\n");exit(1);}
        
        read_bad_lst(X1, Y1, X2, Y2, &N_bad_regions);
        double star_x, star_y;
      
	A=malloc(MAX_NUMBER_OF_STARS*sizeof(double));
	if( A==NULL ){fprintf(stderr,"ERROR: out of memory in function autodetect_aperture() A !!!\n");exit(1);}

	/* Set fixed aperture size if we whant to use it */
	if( fixed_aperture!=0.0 )
	 APERTURE=fixed_aperture;
	else{
         /* Calculate best aperture size from seeing */
	 sprintf(sextractor_catalog_filename,"autodetect_aper_%d.cat",pid);
	 sprintf(command, "sex -c default.sex -PARAMETERS_NAME autodetect_aper.param -CATALOG_NAME %s  %s",sextractor_catalog_filename,fitsfilename);
	 fprintf(stderr, "%s\n", command);
	 system(command);
	 FILE *catalog;
	 catalog = fopen(sextractor_catalog_filename, "r");
	 if(catalog==NULL){fprintf(stderr,"ERROR: cannot open %s\n",sextractor_catalog_filename);exit(1);}
	 while (-1 < fscanf(catalog, "%lf %d %lf %lf", &A[i], &sextractor_flag, &star_x, &star_y)) {
	         if( sextractor_flag==0 && A[i]>0.9 && 0==exclude_region( X1, Y1, X2, Y2, N_bad_regions, star_x, star_y, 5.0) && star_x>OTSTUP && star_y>OTSTUP && fabs(star_x-X_im_size)>OTSTUP && fabs(star_y-Y_im_size)>OTSTUP )
	 	 i++;
	   	 if ( i>=MAX_NUMBER_OF_STARS ){
		  fprintf(stderr, "Oops!!! Too many stars!\nChange string \"#define MAX_NUMBER_OF_STARS %d\" in src/limits.h file and recompile the program by running \"make\".\n",
		   	MAX_NUMBER_OF_STARS);
		  exit(1);
		 } 
	 }
	 fclose(catalog);

         // remove the SExtractor catalog from the 1st pass with wich we measured sizes of stars, we don't need it anymore
         if(0!=unlink(sextractor_catalog_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",sextractor_catalog_filename);
          
	 gsl_sort(A,1,i);
	 median_A=gsl_stats_median_from_sorted_data(A,1,i);
	 APERTURE = median_A * CONST;
	} //if( fixed_aperture!=0.0 )

        // Save aperture information
        aperture_file=fopen(aperture_filename,"w");
        if(aperture_file==NULL){fprintf(stderr,"ERROR: cannot open for writing %s\n",aperture_filename);exit(1);}
        fprintf(aperture_file,"%.1lf",APERTURE);
        fclose(aperture_file);

	
	if (param_P == 0) {
		sprintf(command, "sex -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", APERTURE, output_sextractor_catalog, fitsfilename);
		fprintf(stderr, "%s\n", command);
		system(command);
	}
	if (param_P == 1) {

                char psfex_param_filename[512];
                sprintf(psfex_param_filename,"psfex_%d.param",pid);
		sprintf(command, "echo \"VIGNET(%.0lf,%.0lf)\" > %s", APERTURE+0.5,APERTURE+0.5, psfex_param_filename);
		system(command);
		sprintf(command, "echo \"X_IMAGE\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"Y_IMAGE\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUX_RADIUS\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUX_MAX\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUX_APER(1)\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUXERR_APER(1)\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"ELONGATION\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLAGS\" >> %s", psfex_param_filename);
		system(command);


	        sprintf(sextractor_catalog_filename,"psfex_input_%d.cat",pid);
	        sprintf(psf_filename,"psfex_input_%d.psf",pid);
		sprintf(command, "sex -c bright_star_blend_check_3.0.sex -PARAMETERS_NAME %s -CATALOG_TYPE FITS_LDAC -CATALOG_NAME %s -PHOT_APERTURES %.1lf %s", psfex_param_filename, sextractor_catalog_filename, APERTURE, fitsfilename);
		fprintf(stderr, "%s\n", command);
		system(command);
		if(0!=unlink(psfex_param_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",psfex_param_filename);
		
		sprintf(command, "psfex %s &>/dev/null",sextractor_catalog_filename);
		fprintf(stderr, "%s\n", command);
		system(command);
		if(0!=unlink(sextractor_catalog_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",sextractor_catalog_filename);

		sprintf(command, "sex -c default.sex -PARAMETERS_NAME psfex_sextractor_2nd_pass.param -PSF_NAME %s -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", psf_filename, APERTURE, output_sextractor_catalog, fitsfilename);
		fprintf(stderr, "%s\n", command);
		system(command);
		if(0!=unlink(psf_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",psf_filename);

	}
	
	free(A);
	
	free(X1);
	free(Y1);
	free(X2);
	free(Y2);
	
	return APERTURE;
}
