#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_statistics_float.h>
#include <gsl/gsl_fit.h>
#include <gsl/gsl_sort_double.h>
#include <gsl/gsl_sort_float.h>
#include <libgen.h> /* for basename() */
#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

// all these are for wait3()
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "cpgplot.h"

#include "setenv_local_pgplot.h"

#include "limits.h"

void print_help(){
 fprintf(stderr,"\n");
 fprintf(stderr,"  --*** HOW TO USE THE LIGHTCURVE PLOTTER ***--\n");
 fprintf(stderr,"\n");
 fprintf(stderr,"  'H' - display this help message\n");
 fprintf(stderr,"  right mouse click - exit\n");
 fprintf(stderr,"  left mouse click - display image corresponding to the data point closest to the cursor (only works if the lightcurve is in VaST format)\n");
 fprintf(stderr,"  'Z' + draw a rectangle to zoom in\n");
 fprintf(stderr,"  'D' or 'Z'+'Z' - default zoom\n");
 fprintf(stderr,"  'E' - display error bars (on/off)\n");
 fprintf(stderr,"  'S' - save lightcurve to .ps file\n");
// fprintf(stderr,"  'N' - save lightcurve to .png file\n");
 fprintf(stderr,"  'T' - terminate a single point\n");
 fprintf(stderr,"  'C' + draw a rectangle - remove many point\n");
 fprintf(stderr,"  '1' - display linear trend fit to the lightcurve (on/off)\n");
 fprintf(stderr,"  '0' - fix linear trend inclination to 0 (on/off). Useful for correcting jumps.\n");
 fprintf(stderr,"  'B' - set a break point for a trend\n");
 fprintf(stderr,"  '-' - subtract trend from the lightcurve\n");
 fprintf(stderr,"  'W' - write edited lightcurve to a data file (will be in the same format as input file)\n");
 fprintf(stderr,"  'K' - time of minimum determination using KvW method. You'll need to specify the eclipse duration with two clicks.\n");
 fprintf(stderr,"  \033[0;36m'U'\033[00m - Try to \033[0;36midentify the star\033[00m\n"); 
 fprintf(stderr,"  \033[0;36m'L'\033[00m - Start web-based \033[0;36mperiod search tool\033[00m\n");
 fprintf(stderr,"\n");
 return;
}

void minumum_kwee_van_woerden(float *fit_jd, float *fit_mag, int fit_n){
 FILE *tmp_lc_file;
 int i;
 tmp_lc_file=fopen("lightcurve_for_kwee_van_woerden.tmp","w");
 if( tmp_lc_file==NULL ){
  fprintf(stderr,"ERROR: cannot open the temporary lightcurve file for writing! Aborting minimum determination routine...\n");
  return;
 }
 for(i=0;i<fit_n;i++){
  fprintf(tmp_lc_file,"%f %f\n",fit_jd[i],fit_mag[i]);
 }
 fclose(tmp_lc_file);
 system("lib/kwee-van-woerden < lightcurve_for_kwee_van_woerden.tmp");
 system("rm -f lightcurve_for_kwee_van_woerden.tmp");
 return;
}

void remove_linear_trend(float *fit_jd, float *mag, int N, double A, double B, double mean_jd, double mean_mag, float jd_min, float jd_max){
 int i;
 float *plot_jd;
 float *plot_y;
 float E1,E2;
 plot_jd=malloc(N*sizeof(float));
 plot_y=malloc(N*sizeof(float));
 for(i=0;i<N;i++){
  if( fit_jd[i]>=jd_min && fit_jd[i]<=jd_max ){
   plot_jd[i]=fit_jd[i]-(float)(mean_jd);
   plot_y[i]=(float)(A)*plot_jd[i]+(float)(B);
   plot_jd[i]=plot_jd[i]+(float)mean_jd;
   if( mag[i]>100.0 )
    mag[i]=mag[i]-plot_y[i]; // assume it's not magnitude but something linear
   else{
    // subtract magnitudes
    E1=powf(10.0f,-0.4f*plot_y[i]);
    E2=powf(10.0f,-0.4f*mag[i]);
    mag[i]=2.5f*log10f(E1/E2);
   }
  } // if( fit_jd[i]>jd_min && fit_jd[i]<jd_max )   
 }

 // correct for mean_mag
 for(i=0;i<N;i++){
  if( fit_jd[i]>=jd_min && fit_jd[i]<=jd_max ){
   if( mag[i]>100.0 )
    mag[i]=mag[i]-mean_mag; // assume it's not magnitude but something linear
   else{
    // subtract magnitudes
    E1=powf(10.0f,-0.4f*mean_mag);
    E2=powf(10.0f,-0.4f*mag[i]);
    mag[i]=2.5f*log10f(E1/E2);
   }
  } // if( fit_jd[i]>jd_min && fit_jd[i]<jd_max )   
 }

 free(plot_y);
 free(plot_jd);

 return;
}

void plot_linear_trend(float *fit_jd, int N, double A, double B, double mean_jd, double mean_mag){
 int i;
 float *plot_jd;
 float *plot_y;
 plot_jd=malloc(N*sizeof(float));
 plot_y=malloc(N*sizeof(float));
 for(i=0;i<N;i++){
  plot_jd[i]=fit_jd[i]-(float)(mean_jd);
  plot_y[i]=(float)(A)*plot_jd[i]+(float)(B);
  plot_jd[i]=plot_jd[i]+(float)mean_jd;
  plot_y[i]=plot_y[i]+(float)mean_mag;
 }
 cpgsci(3);
 cpgline(N, plot_jd, plot_y);
 free(plot_y);
 free(plot_jd);
 return;
}

void fit_linear_trend(float *input_JD, float *input_mag, float *mag_err, int N, double *A, double *B, double *mean_jd, double *mean_mag){
// double cov00,cov01; // needed if we want to use gsl_fit_wlinear() instead of gsl_fit_wmul()
 double cov11,chisq;
 double *fit_jd;
 double *fit_mag;
 double *fit_w;
 double *difference;
 int i;
 fit_jd=malloc(N*sizeof(double));
 fit_mag=malloc(N*sizeof(double));
 fit_w=malloc(N*sizeof(double));
 difference=malloc(N*sizeof(double));
 
 for(i=0;i<N;i++){
  fit_jd[i]=(double)input_JD[i];
  fit_mag[i]=(double)input_mag[i];
  fit_w[i]=1.0/(double)(mag_err[i]*mag_err[i]);
 }
 (*mean_jd)=gsl_stats_mean(fit_jd,1,N);
 (*mean_mag)=gsl_stats_mean(fit_mag,1,N);
 for(i=0;i<N;i++){
  fit_jd[i]=fit_jd[i]-(*mean_jd);
  fit_mag[i]=fit_mag[i]-(*mean_mag);
 }
 //gsl_fit_wlinear(fit_jd,1,fit_w,1,fit_mag,1,N,B,A,&cov00,&cov01,&cov11,&chisq);
 gsl_fit_wmul(fit_jd,1,fit_w,1,fit_mag,1,N,A,&cov11,&chisq);
 (*B)=0.0;
 
 fprintf(stderr,"linear trend fit:   %lf mag/day\n", (*A));

 free(difference);
 free(fit_w);
 free(fit_mag);
 free(fit_jd);
 return;
}

 
void fit_median_for_jumps(float *input_JD, float *input_mag, float *mag_err, int N, double *A, double *B, double *mean_jd, double *mean_mag){
 double *fit_jd;
 double *fit_mag;
 double *fit_w;
 double *difference;
 int i;
 fit_jd=malloc(N*sizeof(double));
 fit_mag=malloc(N*sizeof(double));
 fit_w=malloc(N*sizeof(double));
 difference=malloc(N*sizeof(double));
 
 for(i=0;i<N;i++){
  fit_jd[i]=(double)input_JD[i];
  fit_mag[i]=(double)input_mag[i];
  fit_w[i]=1.0/(double)(mag_err[i]*mag_err[i]);
 }
 (*mean_jd)=gsl_stats_mean(fit_jd,1,N);
 gsl_sort(fit_mag,1,N);
 (*mean_mag)=gsl_stats_median_from_sorted_data(fit_mag,1,N);
 (*A)=0.0;
 (*B)=0.0;
 
 fprintf(stderr,"median magnitude:   %lf\n", (*mean_mag));

 free(difference);
 free(fit_w);
 free(fit_mag);
 free(fit_jd);
 return;
}

int get_star_number_from_name(char *output_str, char *input_str){
 char str1[FILENAME_LENGTH];
 char *str;
 int i;
 strcpy(str1,input_str);
 str=basename(str1);
 // if file name is too short
 if( strlen(str)<8 ){
  strcpy(output_str," ");
  return 0;
 }
 str[strlen(str)-4]='\0'; // remove ".dat"
 // remove 'out' if it is part of the name
 if( str[0]=='o' && str[1]=='u' && str[2]=='t' ){
  for(i=0;i<strlen(str)-2;i++)
   str1[i]=str[i+3];
 }
 else{
  strcpy(output_str,str);
  return 1;
 }
 strcpy(output_str,str1);
 return atoi(str1);
}

int find_closest(float x, float y, float *X, float *Y,int N, float new_X1, float new_X2, float new_Y1, float new_Y2){
 float y_to_x_scaling_factor=fabsf(new_X2-new_X1)/fabsf(new_Y2-new_Y1);
 int i;
 float best_dist;
 int best_dist_num=0;
 best_dist=(x-X[0])*(x-X[0])+(y-Y[0])*(y-Y[0])*y_to_x_scaling_factor*y_to_x_scaling_factor; //!!
 for(i=1;i<N;i++){
  if( (x-X[i])*(x-X[i])+(y-Y[i])*(y-Y[i])*y_to_x_scaling_factor*y_to_x_scaling_factor<best_dist ){
   best_dist=(x-X[i])*(x-X[i])+(y-Y[i])*(y-Y[i])*y_to_x_scaling_factor*y_to_x_scaling_factor;
   best_dist_num=i;
  }
 } 
 return best_dist_num;
}

int is_comment(char *str){
 int i;
 int is_empty=1;
 int n=strlen(str);
 
 if( n<1 )return 1;
 
 for(i=0;i<n-1;i++){
  if( str[i]!=' ' && str[i]!='0' && str[i]!='1' && str[i]!='2' && str[i]!='3' && str[i]!='4' && str[i]!='5' && str[i]!='6' && str[i]!='7' && str[i]!='8' && str[i]!='9' && str[i]!='.' && str[i]!='\r' && str[i]!='\n' && str[i]!='\t' && str[i]!='+' && str[i]!='-' )return 1;
  if( str[i]=='\t' )str[i]=' ';
  if( str[i]=='\r' )str[i]=' ';
  if( str[i]!=' ' )is_empty=0;
 }

 if( is_empty==1 )return 1;

 return 0;
}                 

int main( int argc, char **argv){
 char strmusor[DATA_PARSER_C_MAX_STR_LENGTH];
 char lightcurvefilename[FILENAME_LENGTH];
 FILE *lightcurvefile;
 double *JD=NULL;
 float  *mag=NULL;
 float  *mag_err=NULL;
 float  *float_JD=NULL;
 float  *X=NULL;
 float  *Y=NULL;
 float  *APER=NULL;
 char **filename;
 int Nobs=0;
 int i,j;
 float minJD,maxJD,minmag,maxmag;
 /* PGPLOT vars */
 float curX,curY;
 float curX2,curY2;
 char curC;
 char PGPLOT_CONTROL[100];
 int change_limits_trigger=0; // 0 - first time draw
                              // 1 - zoom in
                              // 2 - redraw plot with default limits
 float new_X1,new_X2,old_X1,old_X2;
 float new_Y1,new_Y2,old_Y1,old_Y2;
 char pokaz_start[1024];
 int xw_ps=0; // default plotting device - /xw
 int exit_after_plot=0; // if =1 - plot once and exit
 int draw_errorbars=0; // draw errorbars? 0 = no
 int was_lightcurve_changed=0; // =1 - it was
 int write_edited_lightcurve_to_file=0; // 1 - do it now, 'W' or 'P' was pressed!
 int start_pokaz_script=0; // 1 - start script, 'P' was pressed!
 int plot_linear_trend_switch=0;
 double A, B, mean_jd, mean_mag; // for trend fitting
 float breaks[512]; // breaks - trends on different sides of a break may be different
 int n_breaks=0; // number of breaks
 float tmp_x[2]; // to draw vertical lines for breaks
 float tmp_y[2]; // to draw vertical lines for breaks
 float *fit_jd=NULL; // for trend fitting 
 float *fit_mag=NULL; // for trend fitting
 float *fit_mag_err=NULL; // for trend fitting
 int fit_n=0; // for trend fitting
 int jump_instead_of_break=0; // = 1 - fix A=0.0 in all linear trenf fits and use median magnitude instead of mean magnitude
 int plot_minimum_fitting_boundaries=0;
 /* ----------- */
 int closest_num;
 int lightcurve_format;
 /* Statistic */
 double JD_first, JD_last;
 float m_mean, sigma, mean_sigma, error_mean;
 char header_str[512];
 char header_str2[512];
 double double_JD2float_JD=0.0;
 char JD_label_str[512];
 char mag_label_str[512];
 char star_name[512];
 
 new_X1=0.0;
 
 int debug_mode=0; // 1 - print debug messages
 
 int status=0; // for wait3()
 
 //char bin_dir_name[512]="./";
 int use_ds9_instead_of_pgfv=0;
 /* Options for getopt() */
 int n;
 //char *cvalue = NULL;
 const char* const shortopt = "9dsneb:";
 const struct option longopt[] = {
  {"ds9", 0, NULL, '9'}, {"debug", 0, NULL, 'd'}, {"save", 0, NULL, 's'}, {"png", 0, NULL, 'n'}, {"errorbars", 0, NULL, 'e'},{"bindir", 1, NULL, 'b'},  {NULL, 0, NULL, 0}
 }; //NULL string must be in the end                                 
 int nextopt;
 //struct stat buf;
 while (nextopt = getopt_long(argc, argv, shortopt, longopt, NULL), nextopt != -1) {
   switch (nextopt) {
    case '9':
     use_ds9_instead_of_pgfv=1;
     fprintf(stderr,"option -9: DS9 will be used as a FITS image viewer\n");
     break;
    case 'd':
     debug_mode=1;
     fprintf(stderr,"option -d: Debug mode activated.\n");
     break;
    case 's':
     xw_ps=1;
     exit_after_plot=1;
     fprintf(stderr,"option -s: lightcurve will be plotted to a .ps file\n");
     break;
    case 'n':
     xw_ps=2;
     exit_after_plot=1;
     fprintf(stderr,"option -n: lightcurve will be plotted to a .png file\n");
     break;
    case 'e':
     draw_errorbars=1;
     fprintf(stderr,"option -e: plot errorbars\n");
     break;
    case  -1:
     fprintf(stderr, "That's all with options\n");
     break;
   } 
 }
 
 if( debug_mode==1 )fprintf(stderr,"Done parsing command line arguments.\n");
 
 if( argc==1 ){
  fprintf(stderr,"Usage:\n ./lc FILENAME\nor\n");
  fprintf(stderr,"./lc FILENAME -9    # to use ds9 FITS viewer instead of pgfv\n");
  fprintf(stderr,"./lc FILENAME -s    # plot lightcurve to a .ps file\n");
  fprintf(stderr,"./lc FILENAME -n    # plot lightcurve to a .png file\n");
  fprintf(stderr,"./lc FILENAME -e    # plot errorbars\n");
  exit(1);
 }
 for (n = optind; n < argc; ++n) {
  strcpy(lightcurvefilename,argv[n]);
 }
 
 fprintf(stderr,"Opening \E[34;47m %s \E[33;00m ...  ",lightcurvefilename);
 lightcurvefile=fopen(lightcurvefilename,"r");
 if( NULL==lightcurvefile ){
  fprintf(stderr,"ERROR: cannot open file!\n");
  exit(1);
 }
 if( NULL==fgets(strmusor,DATA_PARSER_C_MAX_STR_LENGTH,lightcurvefile) ){
  fprintf(stderr,"ERROR: empty lightcurve file!\n");
  exit(1);
 }
 
 if( 1==is_comment(strmusor) ){
  while( NULL!=fgets(strmusor,1000,lightcurvefile) ){
   if( 4==sscanf(strmusor,"%lf %f %f %f",&JD_first,&m_mean,&sigma,&mean_sigma) )break; // VaST lightcurve format
   if( 0==is_comment(strmusor) )break;
  }
 }
 
 /* Identify lightcurve format */
 if( 2==sscanf(strmusor,"%lf %f",&JD_first,&m_mean) ){
  lightcurve_format=2; // "JD mag" format
  if( 3==sscanf(strmusor,"%lf %f %f",&JD_first,&m_mean,&sigma) ){
   lightcurve_format=1; // "JD mag err" format
   if( 4==sscanf(strmusor,"%lf %f %f %f",&JD_first,&m_mean,&sigma,&mean_sigma) )
    lightcurve_format=0; // VaST lightcurve format
  }
 }
 else{
  fprintf(stderr,"ERROR: cannot parse the lightcurve file!\n");
  exit(1);
 }
 if( lightcurve_format==0 )fprintf(stderr,"VaST lightcurve format detected!\n");
 if( lightcurve_format==1 )fprintf(stderr,"\"JD mag err\" lightcurve format detected!\n");
 if( lightcurve_format==2 )fprintf(stderr,"\"JD mag\" lightcurve format detected!\n");

 /* Compute number of lines and allocate memory */
 fseek(lightcurvefile,0,SEEK_SET); // go back to the beginning of the lightcurve file
 for(i=0;;i++)
  if( NULL==fgets(strmusor,TESTOR_MAX_MUSOR,lightcurvefile) )
   break;
 fprintf(stderr,"%d lines in the lightcurve file %s\n",i,lightcurvefilename);
 if( lightcurve_format==0 )filename=(char **)malloc((i+1)*sizeof(char **));//malloc((i+1)*sizeof(char **));
 fseek(lightcurvefile,0,SEEK_SET); // go back to the beginning of the lightcurve file
 
 
 //fprintf(stderr,"reading \E[34;47m %s \E[33;00m ... ",lightcurvefilename); 
 /* Read the lightcurve assumming the appropriate format... */
 if( lightcurve_format==0 ){ 
  do{
   JD=realloc(JD,(Nobs+1)*sizeof(double));
   mag=realloc(mag,(Nobs+1)*sizeof(float));
   mag_err=realloc(mag_err,(Nobs+1)*sizeof(float));
   X=realloc(X,(Nobs+1)*sizeof(float));
   Y=realloc(Y,(Nobs+1)*sizeof(float));
   APER=realloc(APER,(Nobs+1)*sizeof(float));
   filename[Nobs]=(char *)malloc(FILENAME_LENGTH*sizeof(char));
   Nobs++;
  }while( -1<fscanf(lightcurvefile,"%lf %f %f %f %f %f %s",&JD[Nobs-1],&mag[Nobs-1],&mag_err[Nobs-1],&X[Nobs-1],&Y[Nobs-1],&APER[Nobs-1],filename[Nobs-1]) );
  Nobs--;
 }
 if( lightcurve_format==1 ){
  JD=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
  mag=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(float));
  mag_err=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(float));
  while( NULL!=fgets(strmusor,DATA_PARSER_C_MAX_STR_LENGTH,lightcurvefile) ){
   if( 1==is_comment(strmusor) )continue;
   sscanf(strmusor,"%lf %f %f", &JD[Nobs], &mag[Nobs], &mag_err[Nobs] );
   Nobs++;
  }
 } 
 if( lightcurve_format==2 ){
  JD=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
  mag=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(float));
  mag_err=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(float));
  while( NULL!=fgets(strmusor,DATA_PARSER_C_MAX_STR_LENGTH,lightcurvefile) ){
   if( 1==is_comment(strmusor) )continue;
   sscanf(strmusor,"%lf %f", &JD[Nobs], &mag[Nobs]);
   mag_err[Nobs]=0.01; //just set some value
   Nobs++;
  }
 } 
// Nobs--;
 fclose(lightcurvefile);
 //fprintf(stderr,"Ok\n");
 
 print_help();
 
 //fprintf(stderr,"\n[%s] Press 'H' or 'I' for help...\n\n",argv[0]);


 /* Searching min max for double (for stats only) */
 JD_first=JD_last=JD[0]; 
 for(i=0;i<Nobs;i++){
  if( JD[i]>JD_last )JD_last=JD[i];
  if( JD[i]<JD_first )JD_first=JD[i];
 }

 /* Determine plot limits */
 minmag=maxmag=mag[0];
 minJD=maxJD=(float)JD[0];
 for(i=1;i<Nobs;i++){
  minmag=MIN(minmag,mag[i]);
  maxmag=MAX(maxmag,mag[i]);
  minJD=MIN(minJD,(float)JD[i]);
 }
 // If possible, do not plot full JD, try something shorter...

 double_JD2float_JD=(double)((int)minJD);

 float_JD=malloc(Nobs*sizeof(float));
 if( float_JD==NULL ){
  fprintf(stderr,"ERROR: can't allocate memory for float_JD: %d*sizeof(float)\n",Nobs);
  return 1;
 }

 minJD=maxJD=(float)(JD[0]-double_JD2float_JD);
 for(i=0;i<Nobs;i++){
  float_JD[i]=(float)(JD[i]-double_JD2float_JD);
  minJD=MIN(minJD,float_JD[i]);
  maxJD=MAX(maxJD,float_JD[i]);
 }

 /* Choose appropriate axes labels */
 if( minmag<0.0 )
  sprintf(mag_label_str,"Instrumental Magnitude");
 else
  sprintf(mag_label_str,"Magnitude");

 if( double_JD2float_JD!=0.0 )
  sprintf(JD_label_str,"JD - %.1lf",double_JD2float_JD);
 else
  sprintf(JD_label_str,"JD");


 /* Start GUI */
 curC=' '; // just some value which doesn't mean anything
 curX=curY=curX2=curY2=0.0; // same here
 //setenv("PGPLOT_DIR","lib/pgplot/",1);
 setenv_localpgplot(argv[0]);
 do{
   // Check what plotting device should be used
   if( xw_ps==1 )strcpy(PGPLOT_CONTROL,"/CPS");
   if( xw_ps==2 )strcpy(PGPLOT_CONTROL,"/PNG");
   if( xw_ps==0 || xw_ps==-1 )strcpy(PGPLOT_CONTROL,"/XW");
 
   if( change_limits_trigger==0 || xw_ps!=0 ){
    cpgopen(PGPLOT_CONTROL);
    if( 0==strcmp(PGPLOT_CONTROL,"/XW") ){
     xw_ps=0;
     if( change_limits_trigger==0 )change_limits_trigger=2;
    }
    if( xw_ps==0 ){
     cpgscr( 0, 0.10, 0.31, 0.32);  /* set default vast window background */
     cpgpage();
    }
    if( xw_ps==2 ){
     cpgscr( 0, 1.0, 1.0, 1.0); // set white background
     cpgscr( 1, 0.0, 0.0, 0.0); // and black foreground
     cpgpage();
    }
   }

   if( xw_ps==0 ){
    cpgscr( 0, 0.10, 0.31, 0.32);  /* set default vast window background */
    cpgeras();
   }

   cpgsvp(0.08, 0.95, 0.1, 0.9);
   if( change_limits_trigger==2 || change_limits_trigger==0 ){
    old_X1=minJD-(maxJD-minJD)/10;
    old_X2=maxJD+(maxJD-minJD)/10;
    new_X1=old_X1;
    new_X2=old_X2;
    
    old_Y1=maxmag+(maxmag-minmag)/10;
    old_Y2=minmag-(maxmag-minmag)/10;
    new_Y1=old_Y1;
    new_Y2=old_Y2;
   }

   cpgswin(new_X1, new_X2, new_Y1, new_Y2);
   if( xw_ps==0 )cpgscr( 0, 0.08, 0.08, 0.09);  /* set background */
   cpgsci(0);
   cpgrect(new_X1, new_X2, new_Y1, new_Y2);
   cpgsci(1);
   cpgscf(1);
   //cpgbox("BCNST1", (new_X2-new_X1)/4.0 , 0, "BCNST1", 0.0, 0);
   cpgbox("BCNST1", 0.0 , 0, "BCNST1", 0.0, 0);

//   fprintf(stderr,"DEBUG: new_X1=%.5f new_X2=%.5f\n",new_X1,new_X2);


   /* Generate some lightcurve stats */
   m_mean=gsl_stats_float_mean(mag,1,Nobs); /* Mean magnitude */
   // if we have information about measurement errors...
   if( lightcurve_format!=2 )
    mean_sigma=gsl_stats_float_mean(mag_err,1,Nobs); /* expected statistical error */
   else
    mean_sigma=-99.9;
   sigma=gsl_stats_float_sd(mag,1,Nobs); /* the standard deviation of the distribution */
   error_mean=sigma/sqrtf((float)Nobs); /* the standard deviation of the mean */

   cpgscf(1);
   cpgsch(1.0); /* make lables with normal characters */
   if( xw_ps==0 )cpgscr( 1, 0.62, 0.81, 0.38); /* set color of lables */
   if( 0!=get_star_number_from_name(star_name,lightcurvefilename) )
    sprintf(header_str,"Star %s, %d observations, start JD=%.4lf end JD=%.4lf", star_name, Nobs, JD_first, JD_last);
   else
    sprintf(header_str,"%d observations, start JD=%.4lf end JD=%.4lf", Nobs, JD_first, JD_last);
   if( strlen(header_str)>78 )sprintf(header_str,"Star %s, %d observations, start JD=%.1lf end JD=%.1lf", star_name, Nobs, JD_first, JD_last);
   if( strlen(header_str)>78 )cpgsch(0.9); /* make lables with small characters */

   // Print second title line only if we are drawing in an X window, not a file
   if( xw_ps==0 ){
    cpgmtxt( "T", 2.0, 0.0, 0.0, header_str);
   
    sprintf(header_str,"<m>=%.3f +/-%.3f,  sigma= %.3f", m_mean, error_mean, sigma);
    if( lightcurve_format!=2 ){
     sprintf(header_str2,", expected sigma = %.3f",mean_sigma);
     strcat(header_str,header_str2);
    }
    cpgmtxt( "T", 1.0, 0.0, 0.0, header_str);
   }
   else{
    // print only one-line title and only if no zoom was used!
    if( change_limits_trigger==2 || change_limits_trigger==0 )
     cpgmtxt( "T", 1.0, 0.0, 0.0, header_str);
   }
   
   /* Make axes lables */
   cpgsch(1.2); /* make axes lables with larger characters */
   cpgmtxt("L",2.5,0.5,0.5,mag_label_str);
   cpgmtxt("B",2.5,0.5,0.5,JD_label_str);
   if( xw_ps==0 )cpgscr( 1, 1.0, 1.0, 1.0); // set color back to normal
   cpgsch(1.0); // set character size back to normal

   if( lightcurve_format!=2 && draw_errorbars==1 ){
    // plot errors
    cpgerrb( 6, Nobs, float_JD, mag, mag_err, 1.0);
   }
   cpgpt(Nobs, float_JD, mag, 17); // plot data

   // plot break lines
   if( n_breaks!=0 )
    gsl_sort_float( breaks, 1, n_breaks);
   cpgsci(4);
   for(i=0;i<n_breaks;i++){
    tmp_x[1]=tmp_x[0]=breaks[i];
    tmp_y[0]=new_Y1;
    tmp_y[1]=new_Y2;
    if( plot_linear_trend_switch==1 )cpgline(2, tmp_x, tmp_y);
    if( plot_minimum_fitting_boundaries==1 ){cpgsci(9);cpgline(2, tmp_x, tmp_y);} // plot minimum fitting boundaries
   }

   
   if( plot_linear_trend_switch==1 ){ // fit and plot linear trends and breaks if there are any, some more details my be found below near remove_linear_trend
    if( fit_n==0 ){
     // allocate memory
     fit_jd=malloc(Nobs*sizeof(float));
     fit_mag=malloc(Nobs*sizeof(float));
     fit_mag_err=malloc(Nobs*sizeof(float));
     fit_n=Nobs;
    }

    if( n_breaks==0 ){
     if( jump_instead_of_break==0 )
      fit_linear_trend( float_JD, mag, mag_err, Nobs, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( float_JD, mag, mag_err, Nobs, &A, &B, &mean_jd, &mean_mag);
     plot_linear_trend( float_JD, Nobs, A, B, mean_jd, mean_mag);
    }

    if( n_breaks==1 ){
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]<=breaks[0] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     plot_linear_trend( fit_jd, fit_n, A, B, mean_jd, mean_mag);
     //
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]>breaks[0] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     plot_linear_trend( fit_jd, fit_n, A, B, mean_jd, mean_mag);
     //
    }

    if( n_breaks>1 ){
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]<=breaks[0] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     plot_linear_trend( fit_jd, fit_n, A, B, mean_jd, mean_mag);
     //
     //
     for(j=1;j<n_breaks;j++){
      fit_n=0;
      for(i=0;i<Nobs;i++){
       if( float_JD[i]>breaks[j-1] && float_JD[i]<breaks[j] ){
        fit_jd[fit_n]=float_JD[i];
        fit_mag[fit_n]=mag[i];
        fit_mag_err[fit_n]=mag_err[i];
        fit_n++;
       }
      }
      if( jump_instead_of_break==0 )
       fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
      else
       fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
      plot_linear_trend( fit_jd, fit_n, A, B, mean_jd, mean_mag);
     }
     //
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]>breaks[n_breaks-1] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     plot_linear_trend( fit_jd, fit_n, A, B, mean_jd, mean_mag);
     //
    }

   } // if( plot_linear_trend_switch==1 ){

   
   cpgsci(5);
   if( xw_ps==0 )cpgcurs(&curX, &curY, &curC);
   /* Zoom */
   if( curC=='Z' || curC=='z' ){
    cpgband( 2, 0, curX, curY, &curX2, &curY2, &curC);
    if( new_X1!=0.0 ){
     old_X1=new_X1;
     old_X2=new_X2;
    }
    new_X1=curX;
    new_Y1=curY;
    
    //fprintf(stderr,"DEBUG: curX=%.5f curY=%.5f curX2=%.5f curY2=%.5f\n",curX,curY,curX2,curY2);

    if( curC=='Z' || curC=='z' )
     curC='D';
    else
     curC='R';
    
    change_limits_trigger=1;
    if( curX2>new_X1 )
     new_X2=curX2;
    else{
     new_X2=new_X1;
     new_X1=curX2;
    }
    if( curY2<new_Y1 )
     new_Y2=curY2;
    else{
     new_Y2=new_Y1;
     new_Y1=curY2;
    }
    
    if( fabs(new_X2-new_X1)<0.05 ){ // do not zoom too much
     old_X1=(new_X1+new_X2)/2.0-0.025;
     old_X2=(new_X1+new_X2)/2.0+0.025;
     new_X1=old_X1;
     new_X2=old_X2;
    }
    if( fabs(new_Y2-new_Y1)<0.005 ){ // do not zoom too much
     old_Y1=(new_Y1+new_Y2)/2.0+0.0025;
     old_Y2=(new_Y1+new_Y2)/2.0-0.0025;
     new_Y1=old_Y1;
     new_Y2=old_Y2;
    }
    
   } /* End of Zoom */
   if( curC=='x' )curC='X'; // in case small 'x' was pressed - that still means exit!

   // Mark that user wants to fit and plot linear trend
   if( curC=='1' ){
    if( plot_linear_trend_switch==0 )
     plot_linear_trend_switch=1;
    else
     plot_linear_trend_switch=0;
   }

   // subtract one or many linear trends (many trends may be defined in regions separated by breaks)   
   if( curC=='-' ){

    if( fit_n==0 ){
     // allocate memory
     fit_jd=malloc(Nobs*sizeof(float));
     fit_mag=malloc(Nobs*sizeof(float));
     fit_mag_err=malloc(Nobs*sizeof(float));
     fit_n=Nobs;
    }

    // if there are no breaks, just one trend
    if( n_breaks==0 ){
     if( jump_instead_of_break==0 )
      fit_linear_trend( float_JD, mag, mag_err, Nobs, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( float_JD, mag, mag_err, Nobs, &A, &B, &mean_jd, &mean_mag);
     remove_linear_trend( float_JD, mag, Nobs, A, B, mean_jd, mean_mag-m_mean, minJD, maxJD);
    }

    // if there is only one break
    if( n_breaks==1 ){
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]<=breaks[0] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     remove_linear_trend( float_JD, mag, Nobs, A, B, mean_jd, mean_mag-m_mean, minJD, breaks[0]);
     //
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]>breaks[0] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     remove_linear_trend( float_JD, mag, Nobs, A, B, mean_jd, mean_mag-m_mean, breaks[0], maxJD);
     //
    }

    // if there are many breaks
    if( n_breaks>1 ){
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]<=breaks[0] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     remove_linear_trend( float_JD, mag, Nobs, A, B, mean_jd, mean_mag-m_mean, minJD, breaks[0]);
     //
     //
     for(j=1;j<n_breaks;j++){
      fit_n=0;
      for(i=0;i<Nobs;i++){
       if( float_JD[i]>breaks[j-1] && float_JD[i]<breaks[j] ){
        fit_jd[fit_n]=float_JD[i];
        fit_mag[fit_n]=mag[i];
        fit_mag_err[fit_n]=mag_err[i];
        fit_n++;
       }
      }
      if( jump_instead_of_break==0 )
       fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
      else
       fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
      remove_linear_trend( float_JD, mag, Nobs, A, B, mean_jd, mean_mag-m_mean, breaks[j-1], breaks[j]);
     }
     //
     //
     fit_n=0;
     for(i=0;i<Nobs;i++){
      if( float_JD[i]>breaks[n_breaks-1] ){
       fit_jd[fit_n]=float_JD[i];
       fit_mag[fit_n]=mag[i];
       fit_mag_err[fit_n]=mag_err[i];
       fit_n++;
      }
     }
     if( jump_instead_of_break==0 )
      fit_linear_trend( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     else
      fit_median_for_jumps( fit_jd, fit_mag, fit_mag_err, fit_n, &A, &B, &mean_jd, &mean_mag);
     remove_linear_trend( float_JD, mag, Nobs, A, B, mean_jd, mean_mag-m_mean, breaks[n_breaks-1], maxJD);
    }

    was_lightcurve_changed=1; // note, that lightcurve was changed
   }
   
   // terminate single data point
   if( curC=='T' || curC=='t' ){
    closest_num=find_closest( curX, curY, float_JD, mag, Nobs, new_X1, new_X2, new_Y1, new_Y2);
    was_lightcurve_changed=1; // note, that lightcurve was changed
    Nobs--;
    for(i=closest_num;i<Nobs;i++){
     JD[i]=JD[i+1];
     float_JD[i]=float_JD[i+1];
     mag[i]=mag[i+1];
     if( lightcurve_format!=2 )mag_err[i]=mag_err[i+1];
     if( lightcurve_format==0 ){
      X[i]=X[i+1];
      Y[i]=Y[i+1];
      APER[i]=APER[i+1];
      strcpy(filename[i],filename[i+1]);
     }
    }
   }
 
   // terminate all data point in selected region
   if( curC=='C' || curC=='c' ){
    cpgsci(2);
    cpgband( 2, 0, curX, curY, &curX2, &curY2, &curC);
    // last chance to cancel!
    if( curC!='X' && curC!='x' ){
     for(closest_num=0;closest_num<Nobs;closest_num++){
      //fprintf(stderr,"%f %f  %f %f\n",MIN(curX,curX2),MAX(curX,curX2),MIN(curY,curY2),MAX(curY,curY2));
      if( float_JD[closest_num]>MIN(curX,curX2) && float_JD[closest_num]<MAX(curX,curX2) && mag[closest_num]>MIN(curY,curY2) && mag[closest_num]<MAX(curY,curY2) ){
       //fprintf(stderr,"Nobs= %d\n",Nobs); // DEBUG!!
       fprintf(stderr,"Removing data point %5d %lf %lf\n",closest_num,JD[closest_num],mag[closest_num]);
       // kill it
       Nobs--;
       for(i=closest_num;i<Nobs;i++){
        //fprintf(stderr,"DEBUG: closest_num=%d    i=%d     Nobs=%d\n",closest_num,i,Nobs);
        //fprintf(stderr,"lightcurve_format=%d JD[%d+1]= %lf\n",lightcurve_format,i,JD[i+1]); // DEBUG!!
        //fprintf(stderr,"                    float_JD[%d+1]= %lf\n",i,float_JD[i+1]); // DEBUG!!
        //fprintf(stderr,"                    mag[%d+1]= %lf\n",i,mag[i+1]); // DEBUG!!
        JD[i]=JD[i+1];
        float_JD[i]=float_JD[i+1];
        mag[i]=mag[i+1];
        if( lightcurve_format!=2 )mag_err[i]=mag_err[i+1];
        if( lightcurve_format==0 ){
         X[i]=X[i+1];
         Y[i]=Y[i+1];
         APER[i]=APER[i+1];
         strcpy(filename[i],filename[i+1]);
        } // if( lightcurve_format==0 )
        // the following works, I don't know why...
        if( closest_num!=-1 )
         closest_num--; /// ! TEST !
       } // for(i=closest_num;i<Nobs;i++) 
       was_lightcurve_changed=1;
      } // if inside the rectangle
     } // for(closest_num=0;i<Nobs;i++)
    } // if( curC!='X' && curC!='x' )
    curC=' ';
   } // if( curC=='C' || curC=='c' )

   // do not allow to write edited lightcurve if no changes were made!
   if( was_lightcurve_changed==0 ){
    if( curC=='W' || curC=='w' ){
     curC=' ';
     fprintf(stderr,"No changes were applied to the lightcurve! I don't want to save it! %%)\n");
    }
   }   
   // write edited lightcurve to file
   if( curC=='W' || curC=='w' ){
    write_edited_lightcurve_to_file=1;
   }

   // start WinEfk wrapper
   if( curC=='P' || curC=='p' ){
    if( was_lightcurve_changed!=0 )write_edited_lightcurve_to_file=1; // write the lightcurve to a new file if it was changed
    start_pokaz_script=1; // and start the script
   } 
   // start web-based period search tool wrapper
   if( curC=='L' || curC=='l' ){
    if( was_lightcurve_changed!=0 )write_edited_lightcurve_to_file=1; // write the lightcurve to a new file if it was changed
    start_pokaz_script=2; // and start the script
   } 
   
   if( write_edited_lightcurve_to_file==1 ){
    write_edited_lightcurve_to_file=0; // don't do it again unless 'W' or 'P' will be pressed again
    if( lightcurvefilename[strlen(lightcurvefilename)-1]=='t' && lightcurvefilename[strlen(lightcurvefilename)-2]=='a' && lightcurvefilename[strlen(lightcurvefilename)-3]=='d' )
     lightcurvefilename[strlen(lightcurvefilename)-4]='\0'; // remove ".dat"
    strcat(lightcurvefilename,"_edit.dat");
    lightcurvefile=fopen(lightcurvefilename,"w");
    for(i=0;i<Nobs;i++){
     fprintf(lightcurvefile, "%.5lf %9.5f",JD[i],mag[i]);
     if( lightcurve_format!=2 )fprintf(lightcurvefile," %.5f",mag_err[i]);
     if( lightcurve_format==0 )fprintf(lightcurvefile," %8.3lf %8.3lf %4.1lf %s",X[i],Y[i],APER[i],filename[i]);
     fprintf(lightcurvefile,"\n");
    }
    fclose(lightcurvefile);
    fprintf(stderr,"Edited lightcurve is written to %s\n",lightcurvefilename);
   }
   if( start_pokaz_script==1 ){
    start_pokaz_script=0; // don't do it again unless 'P' will be pressed again   
    //sprintf(pokaz_start,"%spokaz_winefk.sh %s ",bin_dir_name, lightcurvefilename);
    sprintf(pokaz_start,"./pokaz_winefk.sh %s ", lightcurvefilename);
    // fork before system() so the parent process is not blocked
    if( 0==fork() ){system(pokaz_start);exit(0);}else{wait3(&status, WNOHANG, NULL);}
    //system(pokaz_start);
    change_limits_trigger=1;
   }
   if( start_pokaz_script==2 ){
    start_pokaz_script=0; // don't do it again unless 'P' will be pressed again   
    sprintf(pokaz_start,"./pokaz_laflerkinman.sh %s ", lightcurvefilename);
    // fork before system() so the parent process is not blocked
    if( 0==fork() ){system(pokaz_start);exit(0);}else{wait3(&status, WNOHANG, NULL);}
    //system(pokaz_start);
    change_limits_trigger=1;
   }

   if( curC=='H' || curC=='h' || curC=='I' || curC=='i' )
    print_help(); // print help message


  /*
      Test if we want to set jump_instead_of_break ?
      That will fix A=0.0 in all linear trend fits.
  */
  if( curC=='0' ){
   if( jump_instead_of_break==1 )
    jump_instead_of_break=0;
   else
    jump_instead_of_break=1;
  }


   /* 
      Set break: a discontinuity on a lightcurve. If linear trend fitting will be used, 
      lightcurve on different sides of the brek will be fitted separately.
   */
   if( curC=='B' || curC=='b' ){
    cpgsci(6);
    cpgband( 6, 0, curX, curY, &curX2, &curY2, &curC);
    breaks[n_breaks]=curX2;
    n_breaks++;
    curC=' ';
    plot_linear_trend_switch=1;
   }
   
   /*
      Minimum time determination
   */
   if( curC=='K' || curC=='k' || curC=='M' || curC=='m' ){
    // Determine fit bounadireas
//    n_breaks=0; // just to make sure there will be no interference with jump and brake handling routines
    cpgsci(9);
    cpgband( 6, 0, curX, curY, &curX2, &curY2, &curC);
    breaks[0]=curX2;
   
    // draw first line
    tmp_x[1]=tmp_x[0]=breaks[0];  
    tmp_y[0]=new_Y1;              
    tmp_y[1]=new_Y2;
    cpgline(2, tmp_x, tmp_y);
                   
    cpgband( 6, 0, curX, curY, &curX2, &curY2, &curC);
    breaks[1]=curX2;
    curC='K'; //
    
    // draw second line
    tmp_x[1]=tmp_x[0]=breaks[1];  
    tmp_y[0]=new_Y1;              
    tmp_y[1]=new_Y2;
    cpgline(2, tmp_x, tmp_y);
    
    // Make sure that  breaks[1]>breaks[0]
    if( breaks[1]<breaks[0] ){
     tmp_x[1]=breaks[1];
     breaks[1]=breaks[0];
     breaks[0]=tmp_x[1];
    }

    // Copy the minimum lightcurve to fit_jd/fit_mag array;
    fit_jd=malloc(Nobs*sizeof(float));
    fit_mag=malloc(Nobs*sizeof(float));
    
    fit_n=0;
    for(i=0;i<Nobs;i++){
     if( float_JD[i]>breaks[0] && float_JD[i]<breaks[1] ){
      fit_jd[fit_n]=float_JD[i];
      fit_mag[fit_n]=mag[i];
      fit_n++;
     }
    }
    fprintf(stderr,"Starting the minimum determination routine...\n");
    fprintf(stderr,"Minimum search boundaries:  %f  %f\n", breaks[0], breaks[1]); // Print the minimum search boundaries
    fprintf(stderr,"Minimum duration (from the above boundaries): %f\n", breaks[1]-breaks[0]);
    minumum_kwee_van_woerden(fit_jd, fit_mag, fit_n); // do actual fitting and print results to a terminal
    // clean-up
    free(fit_mag);
    free(fit_jd);
    fit_n=0;
    
    n_breaks=2; // to draw fitting boundaries again after the screen was wiped
    plot_minimum_fitting_boundaries=1;
   }
   

   // Note that we want to draw errorbars
   if( curC=='E' || curC=='e' ){
    if( draw_errorbars==0 )
     draw_errorbars=1;
    else
     draw_errorbars=0;
   }
   
   if( curC=='S' || curC=='s' )xw_ps=1; // save picture to .ps file
   
   if( curC=='N' || curC=='n' )xw_ps=2; // save picture to .png file
   
   if( curC=='U' || curC=='u' ){ // start util/identify.sh 
    //sprintf(strmusor,"util/identify.sh %s &>/dev/null",lightcurvefilename);
    sprintf(strmusor,"util/identify.sh %s",lightcurvefilename);
    fprintf(stderr,"%s\n",strmusor);
    // fork before system() so the parent process is not blocked     
    if( 0==fork() ){system(strmusor);exit(0);}else{wait3(&status, WNOHANG, NULL);}
   }
   
   
   // Set default zoom
   if( curC=='D' || curC=='d' ){
     change_limits_trigger=2;
   }
   
   /* 
      Left click - if the lightcuve is in VaST format - open image corresponding to the closest data point.
   */
   if( curC=='A' || curC=='a' ){
    change_limits_trigger=1;
    closest_num=find_closest( curX, curY, float_JD, mag, Nobs, new_X1, new_X2, new_Y1, new_Y2);
    fprintf(stderr,"%13.5lf  %.5lf %.5lf\n",JD[closest_num],mag[closest_num],mag_err[closest_num]); // Print the selected data point
    //fprintf(stderr,"%13.5f  %.5lf %.5lf -- DEBUG\n",float_JD[closest_num],mag[closest_num],mag_err[closest_num]);
    if( lightcurve_format==0 ){
     if( NULL!=filename[closest_num] ){
      if( use_ds9_instead_of_pgfv==1 )
       sprintf(strmusor,"util/draw_stars_with_ds9.sh %s %.1f %.1f %.1f %s >/dev/null",filename[closest_num],X[closest_num],Y[closest_num],APER[closest_num],lightcurvefilename);
      else
       sprintf(strmusor,"./pgfv %s %.1f %.1f %.1f",filename[closest_num],X[closest_num],Y[closest_num],APER[closest_num]);
      fprintf(stderr,"%s\n",strmusor);
      // fork before system() so the parent process is not blocked
      if( 0==fork() ){system(strmusor);exit(0);}else{wait3(&status, WNOHANG, NULL);}
     }
     else
      fprintf(stderr,"Oops! There is no file!\n");
    }
    else
     fprintf(stderr,"Oops! The lightcurve is not in VaST format! No information about image corresponding to this data point is available!\n");
   }
   
   /* If we ware plotting to a file instead of X window... */
   if( 0==strcmp(PGPLOT_CONTROL,"/CPS") || 0==strcmp(PGPLOT_CONTROL,"/PNG") ){
    cpgclos();

    if( 0!=get_star_number_from_name(star_name,lightcurvefilename) ){
     if( xw_ps==1 )
      sprintf(strmusor,"mv pgplot.ps %s.ps",star_name);
     if( xw_ps==2 )
      sprintf(strmusor,"mv pgplot.png %s.png",star_name);
     system(strmusor); // rename the file
    }
    else
     strcpy(star_name,"pgplot");

    if( xw_ps==1 )fprintf(stderr,"Lightcurve saved to %s.ps\n",star_name);
    if( xw_ps==2 )fprintf(stderr,"Lightcurve saved to %s.png\n",star_name);

    xw_ps=-1;
    curC=' '; 
   
    if( exit_after_plot==1 )
     break;
   }
   
 }while( curC!='X' );
 
 // Free memory
 free(float_JD);
 free(JD);
 free(mag);
 free(mag_err);
 
 if( lightcurve_format==0 ){ 
  for(i=Nobs+1;i--;)free(filename[i]);
  free(filename);
  free(X);
  free(Y);
  free(APER);
 }
 
 return 0;
}

