#!/bin/bash
#
# This script will conduct astrometric calibration of an image. It may also identify a star using this calibrated image.
#
ERROR_STATUS=0

# Find curl
CURL=`which curl` 

# Find SExtractor
SEXTRACTOR=`which sex 2>/dev/null`
if [ "" == "$SEXTRACTOR" ];then
 SEXTRACTOR=lib/bin/sex
fi

# Determine working mode: star identification or image WCS calibration
START_NAME=`basename $0`
if [ "$START_NAME" == "wcs_image_calibration.sh" ];then
 echo "Entering WCS image calibration mode."
 if [ -z $1 ]; then
  echo "Usage: $0 image.fit"
  echo "or"
  echo "       $0 image.fit FIELD_OF_VIEW_IN_ARCMIN"
  exit
 fi
 FITSFILE=$1
 APP=6.0 
else
 echo "Entering star identification mode."
 if [ -z $1 ]; then
  echo "Usage: $0 outNUMBER.dat"
  echo "or"
  echo "       $0 outNUMBER.dat FIELD_OF_VIEW_IN_ARCMIN"
  exit
 fi
 LIGHTCURVEFILE=$1
 # Get the reference image name from the lightcurve file (first line, last column)
 echo -ne "reading light curve $LIGHTCURVEFILE - "
 read JD MAG MERR X Y APP FITSFILE < $LIGHTCURVEFILE && echo "ok"
 # TRAP!! If we whant to identify a flare, there will be no sence to search for an asteroid on the reference image.
 # Use the first discovery image instead!
 REFERENCE_IMAGE=`cat vast_summary.log |grep "Ref.  image:" | awk '{print $6}'`
 if [ "$START_NAME" == "identify_transient.sh" ];then
  while read JD MAG MERR X Y APP FITSFILE ;do
  if [ "$FITSFILE" != "$REFERENCE_IMAGE" ] ;then 
   break
  fi
  done < $LIGHTCURVEFILE
 fi
fi



# Parse the command line arguments
if [ -z $2 ];then 
 FOV=40
 echo "Field of view was not set. Retreating to a default value $FOV'..."
else
 FOV=$2
fi

# Now the interesting part...

# Tesit if the original image is already a calibrated one
TEST_SUBSTRING=`basename $FITSFILE`
TEST_SUBSTRING=`expr substr $TEST_SUBSTRING  1 4`
if [ "$TEST_SUBSTRING" == "wcs_" ];then
 cp $FITSFILE .
 WCS_IMAGE_NAME=`basename $FITSFILE`
else
 WCS_IMAGE_NAME=wcs_`basename $FITSFILE`
fi


if [ ! -f $WCS_IMAGE_NAME ];then
 echo "No image with WCS calibration found."
 echo -n "Starting SExtractor...  "
 if [ ! -f vast_summary.log ];then
  echo "vast_summary.log not found! Creating a fake one..."
  # Note, we set default.sex.PHOTO instead of default.sex file because the situation when we have
  # a dataset reduced with an old version of vast which does not write the log file is more likely
  # to occur with photographic data
  #echo "SExtractor parameter file: default.sex.PHOTO " > vast_summary.log 
  # No, this was stupid! The data loading script will make sure the proper default.sex file is in the directory!
  echo "SExtractor parameter file: default.sex " > vast_summary.log 
 fi
 #sex -c `grep "SExtractor parameter file:" vast_summary.log |awk '{print $4}'` -PHOT_APERTURES $APP -CATALOG_NAME out$$.xyls -CATALOG_TYPE FITS_1.0 -PARAMETERS_NAME xylist.param $FITSFILE && echo "ok"
 IMAGE_SIZE=`lib/astrometry/get_image_dimentions $FITSFILE | awk '{print "width="$2" -F hight="$4}'`
 # EXPERIMENTAL STUFF 
 lib/make_outxyls_for_astrometric_calibration `lib/fits2cat $FITSFILE` out$$.xyls `lib/astrometry/get_image_dimentions $FITSFILE | awk '{print $2" "$4}'`
 if [ ! -f out$$.xyls ];then
  echo "ERROR: out$$.xyls not found!"
  exit 1
 fi
 echo " 
This service uses tools provided by Astrometry.net.
For more details visit http://astrometry.net/ .

Please note, the automatic identification usually works fine with large field 
of view (say, >40 arcmin). If the images are smaller than 20', the automatic 
field identification have good chances to fail. Sorry... :(

Submitting image identification job to a server. This may take time..."
 echo -e "I mean, \033[01;31mlong time\033[00m..."
 echo "Go, have some tea... ;)"
 # Web-based processing
 $CURL -F file=@out$$.xyls -F submit="Upload Image" -F fov=$FOV -F $IMAGE_SIZE "http://scan.sai.msu.ru/cgi-bin/process_file/process_sextractor_list.py" --user vast48:khyzbaojMhztNkWd > server_reply$$.html
 # Local processing
 #$CURL -F file=@out$$.xyls -F submit="Upload Image" -F fov=$FOV -F $IMAGE_SIZE "http://192.168.1.50/cgi-bin/process_file/process_sextractor_list.py" --user vast48:khyzbaojMhztNkWd > server_reply$$.html
 if [ $? -ne 0 ];then
  echo "An ERROR has occured while uploading the star list to the server!"
  exit 1
 fi
 EXPECTED_WCS_HEAD_URL=`grep WCS_HEADER_FILE= server_reply$$.html |awk '{print $2}'`
 SOLVE_FILE_URL=${EXPECTED_WCS_HEAD_URL//out.wcs/out.solved}
 $CURL "$SOLVE_FILE_URL" --user vast48:khyzbaojMhztNkWd |grep "404 Not Found" >/dev/null
 if [ $? == 1 ];then
  echo -e "Field \033[01;32mSolved\033[00m =)"
  rm -f out.solved
  echo -n "Downloading header...  "
  $CURL "$EXPECTED_WCS_HEAD_URL" -o out$$.wcs --user vast48:khyzbaojMhztNkWd &>/dev/null  && echo "done"
  if [ -f out$$.wcs ];then
   cp $FITSFILE `basename $FITSFILE`
   lib/astrometry/insert_wcs_header out$$.wcs `basename $FITSFILE`
   rm -f `basename $FITSFILE` out$$.wcs
  else
   echo "ERROR: can not download out.wcs "
   ERROR_STATUS=1
  fi
 else
  echo -e "Sadly, the field was \033[01;31mnot solved\033[00m. :("
  echo "Try to set smaller field of view size, for example:  $0 $1 " `echo "$FOV/2"|bc -ql`
  echo "If it doesn't help, try to solve the field manually and use util/vismatch_CCD.sh instead of this script..." 
  ERROR_STATUS=1
 fi
 # At this point we should remove the completed job from the server
 SERVER_JOB_ID=`grep "Job ID:" server_reply$$.html |awk '{print $3}'`
 echo "Sending request to remove job $SERVER_JOB_ID from the server...  "
 VaSTID="XGkbtHTGfTPVLrZLBdtIDPzGAEAjaZWW"
 # Web-based processing
 $CURL -F JobID="$SERVER_JOB_ID" -F VaSTID=$VaSTID "http://scan.sai.msu.ru/cgi-bin/process_file/remove_job.py" --user vast48:khyzbaojMhztNkWd 
 # Local processing
 #$CURL -F JobID="$SERVER_JOB_ID" -F VaSTID=$VaSTID "http://192.168.1.50/cgi-bin/process_file/remove_job.py" --user vast48:khyzbaojMhztNkWd 
 if [ $? -ne 0 ];then
  echo "Hmm... Some error?"
 fi
 rm -f server_reply$$.html out$$.xyls
else
 echo "WCS calibrated image found: $WCS_IMAGE_NAME"
fi

# Check for errors on previous steps
if [ $ERROR_STATUS -ne 0 ];then
 echo "ERROR: $ERROR_STATUS"
 exit
fi

 # At this point, we should somehow have a WCS calibrated image named $WCS_IMAGE_NAME
 SEXTRACTOR_CATALOG_NAME="$WCS_IMAGE_NAME".cat
 if [ ! -f $SEXTRACTOR_CATALOG_NAME ];then
  echo "Starting SExtractor" >/dev/stderr
  echo -ne "Starting SExtractor - "
  $SEXTRACTOR -c `grep "SExtractor parameter file:" vast_summary.log |awk '{print $4}'` -PARAMETERS_NAME wcs.param -CATALOG_NAME $SEXTRACTOR_CATALOG_NAME $WCS_IMAGE_NAME && echo "ok"
  echo "Catalog $SEXTRACTOR_CATALOG_NAME corresponding to the reference image $WCS_IMAGE_NAME created."
 else
  echo "Catalog $SEXTRACTOR_CATALOG_NAME corresponding to the reference image $WCS_IMAGE_NAME found." 
#  echo "Catalog $SEXTRACTOR_CATALOG_NAME corresponding to the reference image $WCS_IMAGE_NAME found."
 fi
 rm -f tmp$$.cat


# If we are in the star identification mode - identify the star!
if [ "$START_NAME" != "wcs_image_calibration.sh" ];then
 while read A B C ;do
  echo out"$A" $B $C >> tmp$$.cat
 done < $SEXTRACTOR_CATALOG_NAME
 STARNUM=`basename $LIGHTCURVEFILE .dat`
 RADEC=`lib/find_star_in_wcs_catalog $X $Y < $SEXTRACTOR_CATALOG_NAME`
  echo " "
  echo "#########################################################################"
  echo "Star:   RA(J2000)   Dec(J2000)    X(pix)   Y(pix)   WCS_calibrated_image"
  ##################################################################################
  # Experimental feature: if you want to use spline correction for astrometry, uncomment
  # the following lines. The spline correction is useful for photographic plates
  # digitized using a flatbed scanner.
  ##################################################################################
  #RADEC_CORRECTED=`util/astrometric_correction.sh $SEXTRACTOR_CATALOG_NAME $RADEC`
  #RADEC=$RADEC_CORRECTED
  ##################################################################################
  #     00148  22:53:44.63 +16:09:08.1   283.894  280.149 wcs_3C454.3_R_20101127_0119.00027019.3C454.3.fts
  echo ${STARNUM//out/" "} `lib/deg2hms $RADEC` $X $Y $WCS_IMAGE_NAME |awk '{printf "%s  %s %s  %8.3f %8.3f %s\n",$1,$2,$3,$4,$5,$6}'
  if [ "$START_NAME" == "identify_for_catalog.sh" ];then
   rm -f tmp$$.cat
   exit 0
  fi
  echo " "
  if [ "$START_NAME" == "identify_transient.sh" ];then
   # This will stop util/search_databases_with_curl.sh from using terminal colors
   util/search_databases_with_curl.sh `lib/deg2hms $RADEC` H
   UNCALIBRATED_IMAGE_NAME=`echo ${WCS_IMAGE_NAME//"wcs_"/}`
   DATEANDTIME=`grep "$UNCALIBRATED_IMAGE_NAME" vast_image_details.log | head -n1 |awk '{print $2" "$3}'`
   util/transients/MPCheck.sh `lib/deg2hms $RADEC` $DATEANDTIME H
  else
   util/search_databases_with_curl.sh `lib/deg2hms $RADEC`
   echo " "
   util/search_databases_with_vizquery.sh `lib/deg2hms $RADEC` ${STARNUM//out/" "}
  fi
  echo "#########################################################################"
  rm -f tmp$$.cat 
 if [ "$START_NAME" == "identify.sh" ];then
  echo -e "The identified star is marked with the cross. Please compare the image with Aladin sky chart to make sure the identification is correct.\n"
  # Write Aladin script
  echo "rm all; get Aladin(POSSII J,JPEG),VizieR(USNO-B1)" `lib/deg2hms $RADEC`" 1'; get Simbad " `lib/deg2hms $RADEC`" 1'; VizieR(2MASS)" `lib/deg2hms $RADEC`" 1'" > Aladin.script
  export PATH=$PATH:$HOME # Aladin is often saved in the home directory
  which Aladin &>/dev/null
  if [ $? -ne 0 ];then
   echo "Here is the Aladin script for you (copy it to Aladin console):"
   echo " "
   cat Aladin.script
   echo " "
   echo "Please note, that you may also put the Aladin executable and the Aladin.jar archive" 
   echo "into your home directory ( $HOME ) to let $0 start Aladin automatically."
  else
   echo "Aladin is starting..."
   Aladin < Aladin.script &>/dev/null &
  fi 
  echo " "
  ./pgfv $WCS_IMAGE_NAME $X $Y
 fi
fi

